﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Net.Http;
using System.Reflection;
using System.Text;
using System.Threading.Tasks;
using Windows.Storage;
using Windows.Storage.Streams;
using Windows.UI.Core;
using Windows.UI.Popups;

namespace RichTextBoxDictionaries
{
    public class SpellCheckingUtil
    {
        public static List<SpellingDictionary> AvailableDictionaries()
        {
            List<SpellingDictionary> dictionaries = new List<SpellingDictionary>();

            dictionaries.Add(new SpellingDictionary()
            {
                Name = "Danish (Denmark)",
                Code = "da-DK",
                FileName = "C1Spell_da-DK.dct",
                Url = "http://demos.componentone.com/dictionaries/C1Spell_da-DK.dct"
            });

            dictionaries.Add(new SpellingDictionary()
            {
                Name = "Dutch (Netherlands)",
                Code = "nl-NL",
                FileName = "C1Spell_nl-NL.dct",
                Url = "http://demos.componentone.com/dictionaries/C1Spell_nl-NL.dct"
            });

            dictionaries.Add(new SpellingDictionary()
            {
                Name = "English (Canada)",
                Code = "en-CA",
                FileName = "C1Spell_en-CA.dct",
                Url = "http://demos.componentone.com/dictionaries/C1Spell_en-CA.dct"
            });

            dictionaries.Add(new SpellingDictionary()
            {
                Name = "English (United Kingdom)",
                Code = "en-GB",
                FileName = "C1Spell_en-GB.dct",
                Url = "http://demos.componentone.com/dictionaries/C1Spell_en-GB.dct"
            });

            dictionaries.Add(new SpellingDictionary()
            {
                Name = "English (United States)",
                Code = "en-US",
                FileName = "C1Spell_en-US.dct",
                Url = "http://demos.componentone.com/dictionaries/C1Spell_en-US.dct"
            });

            dictionaries.Add(new SpellingDictionary()
            {
                Name = "French (Canada)",
                Code = "fr-CA",
                FileName = "C1Spell_fr-CA.dct",
                Url = "http://demos.componentone.com/dictionaries/C1Spell_fr-CA.dct"
            });

            dictionaries.Add(new SpellingDictionary()
            {
                Name = "French (France)",
                Code = "fr-FR",
                FileName = "C1Spell_fr-FR.dct",
                Url = "http://demos.componentone.com/dictionaries/C1Spell_fr-FR.dct"
            });

            dictionaries.Add(new SpellingDictionary()
            {
                Name = "German (Germany)",
                Code = "de-DE",
                FileName = "C1Spell_de-DE.dct",
                Url = "http://demos.componentone.com/dictionaries/C1Spell_de-DE.dct"
            });

            dictionaries.Add(new SpellingDictionary()
            {
                Name = "German (Switzerland)",
                Code = "de-CH",
                FileName = "C1Spell_de-CH.dct",
                Url = "http://demos.componentone.com/dictionaries/C1Spell_de-CH.dct"
            });

            dictionaries.Add(new SpellingDictionary()
            {
                Name = "Greek (Greece)",
                Code = "el-GR",
                FileName = "C1Spell_el-GR.dct",
                Url = "http://demos.componentone.com/dictionaries/C1Spell_el-GR.dct"
            });

            dictionaries.Add(new SpellingDictionary()
            {
                Name = "Italian (Italy)",
                Code = "it-IT",
                FileName = "C1Spell_it-IT.dct",
                Url = "http://demos.componentone.com/dictionaries/C1Spell_it-IT.dct"
            });

            dictionaries.Add(new SpellingDictionary()
            {
                Name = "Norwegian (Bokmål)",
                Code = "nb-NO",
                FileName = "C1Spell_nb-NO.dct",
                Url = "http://demos.componentone.com/dictionaries/C1Spell_nb-NO.dct"
            });

            dictionaries.Add(new SpellingDictionary()
            {
                Name = "Portuguese (Brazil)",
                Code = "pt-BR",
                FileName = "C1Spell_pt-BR.dct",
                Url = "http://demos.componentone.com/dictionaries/C1Spell_pt-BR.dct"
            });

            dictionaries.Add(new SpellingDictionary()
            {
                Name = "Portuguese (Portugal)",
                Code = "pt-PT",
                FileName = "C1Spell_pt-PT.dct",
                Url = "http://demos.componentone.com/dictionaries/C1Spell_pt-PT.dct"
            });

            dictionaries.Add(new SpellingDictionary()
            {
                Name = "Russian (Russia)",
                Code = "ru-RU",
                FileName = "C1Spell_ru-RU.dct",
                Url = "http://demos.componentone.com/dictionaries/C1Spell_ru-RU.dct"
            });

            dictionaries.Add(new SpellingDictionary()
            {
                Name = "Spanish (Argentina)",
                Code = "es-AR",
                FileName = "C1Spell_es-AR.dct",
                Url = "http://demos.componentone.com/dictionaries/C1Spell_es-AR.dct"
            });

            dictionaries.Add(new SpellingDictionary()
            {
                Name = "Spanish (Mexico)",
                Code = "es-MX",
                FileName = "C1Spell_es-MX.dct",
                Url = "http://demos.componentone.com/dictionaries/C1Spell_es-MX.dct"
            });

            dictionaries.Add(new SpellingDictionary()
            {
                Name = "Spanish (Spain)",
                Code = "es-ES",
                FileName = "C1Spell_es-ES.dct",
                Url = "http://demos.componentone.com/dictionaries/C1Spell_es-ES.dct"
            });

            dictionaries.Add(new SpellingDictionary()
            {
                Name = "Swedish (Sweden)",
                Code = "sv-SE",
                FileName = "C1Spell_sv-SE.dct",
                Url = "http://demos.componentone.com/dictionaries/C1Spell_sv-SE.dct"
            });


            return dictionaries;
        }

        public static SpellingDictionary GetSpellingDictionary(string code)
        {
            foreach (SpellingDictionary dictionary in AvailableDictionaries())
            {
                if (dictionary.Code.Equals(code))
                {
                    return dictionary;
                }
            }
            return AvailableDictionaries()[0];
        }

        private static string PreviousDictionaryCode = "";
        public static async void LoadMainDictionary()
        {
            var mainDictionary = App.SpellChecker.MainDictionary;

            // set main dictionary
            mainDictionary.Enabled = true;
            string mainDictionaryCode = ApplicationSettings.MainDictionaryCode();

            // for efficiency do not reload same dictionary
            if (PreviousDictionaryCode.Equals(mainDictionaryCode))
                return;

            PreviousDictionaryCode = mainDictionaryCode;

            // get dictionary object
            SpellingDictionary dictionary = SpellCheckingUtil.GetSpellingDictionary(mainDictionaryCode);

            if (dictionary != null)
            {
                App.ShowLoadingIndicator(true);
                    
                // if english, load from resources
                if (dictionary.Code.Equals("en-US"))
                {
                    Assembly asm = typeof(MainPage).GetTypeInfo().Assembly;
                    Stream stream = asm.GetManifestResourceStream("RichTextBoxDictionaries.Resources.C1Spell_en-US.dct");

                    await mainDictionary.LoadAsync(stream);
                }
                else
                {
                    // locate foreign dictionary from local data storage
                    StorageFile file = null;
                    try
                    {
                        file = await ApplicationData.Current.LocalFolder.GetFileAsync(dictionary.FileName);
                    }
                    catch (Exception ex)
                    {
                        //file not found
                    }

                    if (file != null)
                    {
                        //file exists, load from storage
                        await mainDictionary.LoadAsync(await file.OpenStreamForReadAsync());
                    }
                    else
                    {
                        // file does not exist in storage
                        // create the empty file
                        file = await ApplicationData.Current.LocalFolder.CreateFileAsync(dictionary.FileName, CreationCollisionOption.ReplaceExisting);
                        // load file from the Web
                        HttpClient client = new HttpClient();
                        try
                        {
                            var stream = await client.GetStreamAsync(new Uri(dictionary.Url, UriKind.Absolute));

                            using (Stream outStream = Task.Run(() => file.OpenStreamForWriteAsync()).Result)
                            {
                                stream.CopyTo(outStream);
                                await outStream.FlushAsync();
                            }

                            await mainDictionary.LoadAsync(await file.OpenStreamForReadAsync());

                        }
                        catch (Exception ex)
                        {

                            file.DeleteAsync(StorageDeleteOption.PermanentDelete);
                            ShowDialog("There was an error attempting to download the dictionary. Choose a different dictionary or try again later.", "Error", null);
                        }
                    }
                }
                App.ShowLoadingIndicator(false);
                
            }

            // set main dictionary code to view model so UI can display the name
            MainViewModel vm = App.Current.Resources["mainViewModel"] as MainViewModel;
            if(vm != null)
            {
                vm.MainDictionary = mainDictionaryCode;
            }

            if (mainDictionary.State != C1.Xaml.SpellChecker.DictionaryState.Loaded)
            {

                await ShowDialog("The main dictionary failed to load.", "Warning", null);
            }
        }

        private static Task ShowDialog(string msg, string title, List<IUICommand> commands)
        {
            CoreDispatcher dispatcher = Windows.ApplicationModel.Core.CoreApplication.MainView.CoreWindow.Dispatcher;
            Func<object, Task<bool>> action = null;
            action = async (o) =>
            {
                try
                {
                    if (dispatcher.HasThreadAccess)
                    {
                        MessageDialog dialog = new MessageDialog(msg, title);
                        if (commands != null)
                        {
                            foreach (IUICommand com in commands)
                            {
                                dialog.Commands.Add(com);
                            }
                        }
                        await dialog.ShowAsync();
                    }
                    else
                    {
                        dispatcher.RunAsync(CoreDispatcherPriority.Normal,
                        () => action(o));
                    }
                    return true;
                }
                catch (UnauthorizedAccessException)
                {
                    if (action != null)
                    {
                        Task.Delay(500).ContinueWith(async t => await action(o));
                    }
                }
                return false;
            };
            return action(null);
        }
    }

    public class SpellingDictionary
    {
        public string Name { get; set; }
        public string Code { get; set; }
        public string Url { get; set; }
        public string FileName { get; set; }

    }
}
